# coding=utf-8
from .utils import assert_auth, to_date_str, hashable_lru
from .client import JQDataClient
import sys


@assert_auth
@hashable_lru(maxsize=3)
def alpha_001(enddate, index='all'):
    """
    Inputs:
        enddate: 必选参数，计算哪一天的因子  
        index: 默认参数，股票指数，默认为所有股票'all' 
    Outputs:
        Series：index 为成分股代码，values为对应的因子值
    公式:
    (rank(Ts_ArgMax(SignedPower(((returns < 0) ? stddev(returns, 20) : close), 2.), 5)) - 0.5)
    """
    enddate = to_date_str(enddate)
    func_name = sys._getframe().f_code.co_name
    return JQDataClient.instance().get_alpha_101(**locals())


@assert_auth
@hashable_lru(maxsize=3)
def alpha_002(enddate, index='all'):
    """
    Inputs:
        enddate: 必选参数，计算哪一天的因子  
        index: 默认参数，股票指数，默认为所有股票'all' 
    Outputs:
        Series：index 为成分股代码，values为对应的因子值
    公式:
        (-1\*correlation(rank(delta(log(volume), 2)), rank(((close - open) / open)), 6))
    """
    enddate = to_date_str(enddate)
    func_name = sys._getframe().f_code.co_name
    return JQDataClient.instance().get_alpha_101(**locals())


@assert_auth
@hashable_lru(maxsize=3)
def alpha_003(enddate, index='all'):
    """
    Inputs:
        enddate: 必选参数，计算哪一天的因子  
        index: 默认参数，股票指数，默认为所有股票'all' 
    Outputs:
        Series：index 为成分股代码，values为对应的因子值
    公式:
        (-1\*correlation(rank(open), rank(volume), 10)) 
    """
    enddate = to_date_str(enddate)
    func_name = sys._getframe().f_code.co_name
    return JQDataClient.instance().get_alpha_101(**locals())


@assert_auth
@hashable_lru(maxsize=3)
def alpha_004(enddate, index='all'):
    """
    Inputs:
        enddate: 必选参数，计算哪一天的因子  
        index: 默认参数，股票指数，默认为所有股票'all' 
    Outputs:
        Series：index 为成分股代码，values为对应的因子值
    公式:
        (-1\*Ts_Rank(rank(low), 9))
    """
    enddate = to_date_str(enddate)
    func_name = sys._getframe().f_code.co_name
    return JQDataClient.instance().get_alpha_101(**locals())


@assert_auth
@hashable_lru(maxsize=3)
def alpha_005(enddate, index='all'):
    """
    Inputs:
        enddate: 必选参数，计算哪一天的因子  
        index: 默认参数，股票指数，默认为所有股票'all' 
    Outputs:
        Series：index 为成分股代码，values为对应的因子值
    公式:
        (rank((open - (sum(vwap, 10) / 10)))\*(-1\*abs(rank((close - vwap)))))
    """
    enddate = to_date_str(enddate)
    func_name = sys._getframe().f_code.co_name
    return JQDataClient.instance().get_alpha_101(**locals())


@assert_auth
@hashable_lru(maxsize=3)
def alpha_006(enddate, index='all'):
    """
    Inputs:
        enddate: 必选参数，计算哪一天的因子  
        index: 默认参数，股票指数，默认为所有股票'all' 
    Outputs:
        Series：index 为成分股代码，values为对应的因子值
    公式:
        (-1\*correlation(open, volume, 10))
     """
    enddate = to_date_str(enddate)
    func_name = sys._getframe().f_code.co_name
    return JQDataClient.instance().get_alpha_101(**locals())


@assert_auth
@hashable_lru(maxsize=3)
def alpha_007(enddate, index='all'):
    """
    Inputs:
        enddate: 必选参数，计算哪一天的因子  
        index: 默认参数，股票指数，默认为所有股票'all' 
    Outputs:
        eries：index为成分股代码，values：符合条件的为公式里的数值，不符合条件的为-1
    公式:
        ((adv20 < volume) ? ((-1\*ts_rank(abs(delta(close, 7)), 60))\*sign(delta(close, 7))) : (-1\*1))
    """
    enddate = to_date_str(enddate)
    func_name = sys._getframe().f_code.co_name
    return JQDataClient.instance().get_alpha_101(**locals())


@assert_auth
@hashable_lru(maxsize=3)
def alpha_008(enddate, index='all'):
    """
    Inputs:
        enddate: 必选参数，计算哪一天的因子  
        index: 默认参数，股票指数，默认为所有股票'all' 
    Outputs:
        Series：index 为成分股代码，values为对应的因子值
    公式:
        (-1\*rank(((sum(open, 5)\*sum(returns, 5)) - delay((sum(open, 5)\*sum(returns, 5)), 10))))
    """
    enddate = to_date_str(enddate)
    func_name = sys._getframe().f_code.co_name
    return JQDataClient.instance().get_alpha_101(**locals())


@assert_auth
@hashable_lru(maxsize=3)
def alpha_009(enddate, index='all'):
    """
    Inputs:
        enddate: 必选参数，计算哪一天的因子  
        index: 默认参数，股票指数，默认为所有股票'all' 
    Outputs:
        Series：index 为成分股代码，values为对应的因子值
    公式:
        ((0 < ts_min(delta(close, 1), 5)) ? delta(close, 1) : ((ts_max(delta(close, 1), 5) < 0) ? delta(close, 1) : (-1\*delta(close, 1))))
    """
    enddate = to_date_str(enddate)
    func_name = sys._getframe().f_code.co_name
    return JQDataClient.instance().get_alpha_101(**locals())


@assert_auth
@hashable_lru(maxsize=3)
def alpha_010(enddate, index='all'):
    """
    Inputs:
        enddate: 必选参数，计算哪一天的因子  
        index: 默认参数，股票指数，默认为所有股票'all' 
    Outputs:
        Series：index 为成分股代码，values为对应的因子值
    公式:
        rank(((0 < ts_min(delta(close, 1), 4)) ? delta(close, 1) : ((ts_max(delta(close, 1), 4) < 0) ? delta(close, 1) : (-1\*delta(close, 1)))))
    """
    enddate = to_date_str(enddate)
    func_name = sys._getframe().f_code.co_name
    return JQDataClient.instance().get_alpha_101(**locals())


@assert_auth
@hashable_lru(maxsize=3)
def alpha_011(enddate, index='all'):
    """
    Inputs:
        enddate: 必选参数，计算哪一天的因子  
        index: 默认参数，股票指数，默认为所有股票'all' 
    Outputs:
        Series：index 为成分股代码，values为对应的因子值
    公式:
        ((rank(ts_max((vwap - close), 3)) + rank(ts_min((vwap - close), 3)))\*rank(delta(volume, 3)))
    """
    enddate = to_date_str(enddate)
    func_name = sys._getframe().f_code.co_name
    return JQDataClient.instance().get_alpha_101(**locals())


@assert_auth
@hashable_lru(maxsize=3)
def alpha_012(enddate, index='all'):
    """
    Inputs:
        enddate: 必选参数，计算哪一天的因子  
        index: 默认参数，股票指数，默认为所有股票'all' 
    Outputs:
        Series：index 为成分股代码，values为对应的因子值
    公式:
        (sign(delta(volume, 1))\*(-1 * delta(close, 1)))
    """
    enddate = to_date_str(enddate)
    func_name = sys._getframe().f_code.co_name
    return JQDataClient.instance().get_alpha_101(**locals())


@assert_auth
@hashable_lru(maxsize=3)
def alpha_013(enddate, index='all'):
    """
    Inputs:
        enddate: 必选参数，计算哪一天的因子  
        index: 默认参数，股票指数，默认为所有股票'all' 
    Outputs:
        Series：index 为成分股代码，values为对应的因子值
    公式:
        (-1\*rank(covariance(rank(close), rank(volume), 5)))
    """
    enddate = to_date_str(enddate)
    func_name = sys._getframe().f_code.co_name
    return JQDataClient.instance().get_alpha_101(**locals())


@assert_auth
@hashable_lru(maxsize=3)
def alpha_014(enddate, index='all'):
    """
    Inputs:
        enddate: 必选参数，计算哪一天的因子  
        index: 默认参数，股票指数，默认为所有股票'all' 
    Outputs:
        Series：index 为成分股代码，values为对应的因子值
    公式:
        ((-1\*rank(delta(returns, 3)))\*correlation(open, volume, 10))
    """
    enddate = to_date_str(enddate)
    func_name = sys._getframe().f_code.co_name
    return JQDataClient.instance().get_alpha_101(**locals())


@assert_auth
@hashable_lru(maxsize=3)
def alpha_015(enddate, index='all'):
    """
    Inputs:
        enddate: 必选参数，计算哪一天的因子  
        index: 默认参数，股票指数，默认为所有股票'all' 
    Outputs:
        Series：index 为成分股代码，values为对应的因子值
    公式:
        (-1\*sum(rank(correlation(rank(high), rank(volume), 3)), 3))
    """
    enddate = to_date_str(enddate)
    func_name = sys._getframe().f_code.co_name
    return JQDataClient.instance().get_alpha_101(**locals())


@assert_auth
@hashable_lru(maxsize=3)
def alpha_016(enddate, index='all'):
    """
    Inputs:
        enddate: 必选参数，计算哪一天的因子  
        index: 默认参数，股票指数，默认为所有股票'all' 
    Outputs:
        Series：index 为成分股代码，values为对应的因子值
    公式:
        (-1\*rank(covariance(rank(high), rank(volume), 5)))
    """
    enddate = to_date_str(enddate)
    func_name = sys._getframe().f_code.co_name
    return JQDataClient.instance().get_alpha_101(**locals())


@assert_auth
@hashable_lru(maxsize=3)
def alpha_017(enddate, index='all'):
    """
    Inputs:
        enddate: 必选参数，计算哪一天的因子  
        index: 默认参数，股票指数，默认为所有股票'all' 
    Outputs:
        eries：index为成分股代码，values为对应的因子值，取值有问题的股票对应值为-0.0
    公式:
        (((-1\*rank(ts_rank(close, 10)))\*rank(delta(delta(close, 1), 1)))\*rank(ts_rank((volume / adv20), 5)))
    """
    enddate = to_date_str(enddate)
    func_name = sys._getframe().f_code.co_name
    return JQDataClient.instance().get_alpha_101(**locals())


@assert_auth
@hashable_lru(maxsize=3)
def alpha_018(enddate, index='all'):
    """
    Inputs:
        enddate: 必选参数，计算哪一天的因子  
        index: 默认参数，股票指数，默认为所有股票'all' 
    Outputs:
        Series：index 为成分股代码，values为对应的因子值
    公式:
        (-1\*rank(((stddev(abs((close - open)), 5) + (close - open)) + correlation(close, open, 10))))
    """
    enddate = to_date_str(enddate)
    func_name = sys._getframe().f_code.co_name
    return JQDataClient.instance().get_alpha_101(**locals())


@assert_auth
@hashable_lru(maxsize=3)
def alpha_019(enddate, index='all'):
    """
    Inputs:
        enddate: 必选参数，计算哪一天的因子  
        index: 默认参数，股票指数，默认为所有股票'all' 
    Outputs:
        Series：index 为成分股代码，values为对应的因子值
    公式:
        ((-1\*sign(((close - delay(close, 7)) + delta(close, 7))))\*(1 + rank((1 + sum(returns, 250)))))
    """
    enddate = to_date_str(enddate)
    func_name = sys._getframe().f_code.co_name
    return JQDataClient.instance().get_alpha_101(**locals())


@assert_auth
@hashable_lru(maxsize=3)
def alpha_020(enddate, index='all'):
    """
    Inputs:
        enddate: 必选参数，计算哪一天的因子  
        index: 默认参数，股票指数，默认为所有股票'all' 
    Outputs:
        Series：index 为成分股代码，values为对应的因子值
    公式:
        (((-1\*rank((open - delay(high, 1))))\* rank((open - delay(close, 1))))\* rank((open - delay(low, 1))))
    """
    enddate = to_date_str(enddate)
    func_name = sys._getframe().f_code.co_name
    return JQDataClient.instance().get_alpha_101(**locals())


@assert_auth
@hashable_lru(maxsize=3)
def alpha_021(enddate, index='all'):
    """
    Inputs:
        enddate: 必选参数，计算哪一天的因子  
        index: 默认参数，股票指数，默认为所有股票'all' 
    Outputs:
        eries：index为成分股代码，values为1或-1，取决于满不满足某些因子中的条件
    公式:
        ((((sum(close, 8) / 8) + stddev(close, 8)) < (sum(close, 2) / 2)) ? (-1\* 1) : (((sum(close, 2) / 2) < ((sum(close, 8) / 8) - stddev(close, 8))) ? 1 : (((1 < (volume / adv20)) or ((volume / adv20) == 1)) ? 1 : (-1\* 1))))
    """
    enddate = to_date_str(enddate)
    func_name = sys._getframe().f_code.co_name
    return JQDataClient.instance().get_alpha_101(**locals())


@assert_auth
@hashable_lru(maxsize=3)
def alpha_022(enddate, index='all'):
    """
    Inputs:
        enddate: 必选参数，计算哪一天的因子  
        index: 默认参数，股票指数，默认为所有股票'all' 
    Outputs:
        Series：index 为成分股代码，values为对应的因子值
    公式:
        (-1\* (delta(correlation(high, volume, 5), 5)\* rank(stddev(close, 20))))
    """
    enddate = to_date_str(enddate)
    func_name = sys._getframe().f_code.co_name
    return JQDataClient.instance().get_alpha_101(**locals())


@assert_auth
@hashable_lru(maxsize=3)
def alpha_023(enddate, index='all'):
    """
    Inputs:
        enddate: 必选参数，计算哪一天的因子  
        index: 默认参数，股票指数，默认为所有股票'all' 
    Outputs:
        eries：index为成分股代码，values为对应因子值或0.00，当不满足条件时为0.00
    公式:
        (((sum(high, 20) / 20) < high) ? (-1\* delta(high, 2)) : 0)
    """
    enddate = to_date_str(enddate)
    func_name = sys._getframe().f_code.co_name
    return JQDataClient.instance().get_alpha_101(**locals())


@assert_auth
@hashable_lru(maxsize=3)
def alpha_024(enddate, index='all'):
    """
    Inputs:
        enddate: 必选参数，计算哪一天的因子  
        index: 默认参数，股票指数，默认为所有股票'all' 
    Outputs:
        Series：index 为成分股代码，values为对应的因子值
    公式:
        ((((delta((sum(close, 100) / 100), 100) / delay(close, 100)) < 0.05) or ((delta((sum(close, 100) / 100), 100) / delay(close, 100)) == 0.05)) ? (-1\* (close - ts_min(close, 100))) : (-1\* delta(close, 3)))
    """
    enddate = to_date_str(enddate)
    func_name = sys._getframe().f_code.co_name
    return JQDataClient.instance().get_alpha_101(**locals())


@assert_auth
@hashable_lru(maxsize=3)
def alpha_025(enddate, index='all'):
    """
    Inputs:
        enddate: 必选参数，计算哪一天的因子  
        index: 默认参数，股票指数，默认为所有股票'all' 
    Outputs:
        Series：index 为成分股代码，values为对应的因子值
    公式:
        rank(((((-1\* returns)\* adv20)\* vwap)\* (high - close)))
    """
    enddate = to_date_str(enddate)
    func_name = sys._getframe().f_code.co_name
    return JQDataClient.instance().get_alpha_101(**locals())


@assert_auth
@hashable_lru(maxsize=3)
def alpha_026(enddate, index='all'):
    """
    Inputs:
        enddate: 必选参数，计算哪一天的因子  
        index: 默认参数，股票指数，默认为所有股票'all' 
    Outputs:
        Series：index 为成分股代码，values为对应的因子值
    公式:
        (-1\* ts_max(correlation(ts_rank(volume, 5), ts_rank(high, 5), 5), 3))
    """
    enddate = to_date_str(enddate)
    func_name = sys._getframe().f_code.co_name
    return JQDataClient.instance().get_alpha_101(**locals())


@assert_auth
@hashable_lru(maxsize=3)
def alpha_027(enddate, index='all'):
    """
    Inputs:
        enddate: 必选参数，计算哪一天的因子  
        index: 默认参数，股票指数，默认为所有股票'all' 
    Outputs:
        eries：index为成分股代码，values为1或-1，满足条件为-1，不满足为1
    公式:
        ((0.5 < rank((sum(correlation(rank(volume), rank(vwap), 6), 2) / 2.0))) ? (-1\* 1) : 1)
    """
    enddate = to_date_str(enddate)
    func_name = sys._getframe().f_code.co_name
    return JQDataClient.instance().get_alpha_101(**locals())


@assert_auth
@hashable_lru(maxsize=3)
def alpha_028(enddate, index='all'):
    """
    Inputs:
        enddate: 必选参数，计算哪一天的因子  
        index: 默认参数，股票指数，默认为所有股票'all' 
    Outputs:
        Series：index 为成分股代码，values为对应的因子值
    公式:
        scale(((correlation(adv20, low, 5) + ((high + low) / 2)) - close))
    """
    enddate = to_date_str(enddate)
    func_name = sys._getframe().f_code.co_name
    return JQDataClient.instance().get_alpha_101(**locals())


@assert_auth
@hashable_lru(maxsize=3)
def alpha_029(enddate, index='all'):
    """
    Inputs:
        enddate: 必选参数，计算哪一天的因子  
        index: 默认参数，股票指数，默认为所有股票'all' 
    Outputs:
        Series：index 为成分股代码，values为对应的因子值
    公式:
        (min(product(rank(rank(scale(log(sum(ts_min(rank(rank((-1\*rank(delta((close - 1), 5))))), 2), 1))))), 1), 5) + ts_rank(delay((-1\* returns), 6), 5))
    """
    enddate = to_date_str(enddate)
    func_name = sys._getframe().f_code.co_name
    return JQDataClient.instance().get_alpha_101(**locals())


@assert_auth
@hashable_lru(maxsize=3)
def alpha_030(enddate, index='all'):
    """
    Inputs:
        enddate: 必选参数，计算哪一天的因子  
        index: 默认参数，股票指数，默认为所有股票'all' 
    Outputs:
        Series：index 为成分股代码，values为对应的因子值
    公式:
        (((1.0 - rank(((sign((close - delay(close, 1))) + sign((delay(close, 1) - delay(close, 2)))) + sign((delay(close, 2) - delay(close, 3))))))\* sum(volume, 5)) / sum(volume, 20))
    """
    enddate = to_date_str(enddate)
    func_name = sys._getframe().f_code.co_name
    return JQDataClient.instance().get_alpha_101(**locals())


@assert_auth
@hashable_lru(maxsize=3)
def alpha_031(enddate, index='all'):
    """
    Inputs:
        enddate: 必选参数，计算哪一天的因子  
        index: 默认参数，股票指数，默认为所有股票'all' 
    Outputs:
        Series：index 为成分股代码，values为对应的因子值
    公式:
        ((rank(rank(rank(decay_linear((-1\* rank(rank(delta(close, 10)))), 10)))) + rank((-1\* delta(close, 3)))) + sign(scale(correlation(adv20, low, 12))))
    """
    enddate = to_date_str(enddate)
    func_name = sys._getframe().f_code.co_name
    return JQDataClient.instance().get_alpha_101(**locals())


@assert_auth
@hashable_lru(maxsize=3)
def alpha_032(enddate, index='all'):
    """
    Inputs:
        enddate: 必选参数，计算哪一天的因子  
        index: 默认参数，股票指数，默认为所有股票'all' 
    Outputs:
        Series：index 为成分股代码，values为对应的因子值
    公式:
        (scale(((sum(close, 7) / 7) - close)) + (20\* scale(correlation(vwap, delay(close, 5), 230))))
    """
    enddate = to_date_str(enddate)
    func_name = sys._getframe().f_code.co_name
    return JQDataClient.instance().get_alpha_101(**locals())


@assert_auth
@hashable_lru(maxsize=3)
def alpha_033(enddate, index='all'):
    """
    Inputs:
        enddate: 必选参数，计算哪一天的因子  
        index: 默认参数，股票指数，默认为所有股票'all' 
    Outputs:
        Series：index 为成分股代码，values为对应的因子值
    公式:
        rank((-1\* ((1 - (open / close))^1)))
    """
    enddate = to_date_str(enddate)
    func_name = sys._getframe().f_code.co_name
    return JQDataClient.instance().get_alpha_101(**locals())


@assert_auth
@hashable_lru(maxsize=3)
def alpha_034(enddate, index='all'):
    """
    Inputs:
        enddate: 必选参数，计算哪一天的因子  
        index: 默认参数，股票指数，默认为所有股票'all' 
    Outputs:
        Series：index 为成分股代码，values为对应的因子值
    公式:
        rank(((1 - rank((stddev(returns, 2) / stddev(returns, 5)))) + (1 - rank(delta(close, 1)))))
    """
    enddate = to_date_str(enddate)
    func_name = sys._getframe().f_code.co_name
    return JQDataClient.instance().get_alpha_101(**locals())


@assert_auth
@hashable_lru(maxsize=3)
def alpha_035(enddate, index='all'):
    """
    Inputs:
        enddate: 必选参数，计算哪一天的因子  
        index: 默认参数，股票指数，默认为所有股票'all' 
    Outputs:
        Series：index 为成分股代码，values为对应的因子值
    公式:
        ((Ts_Rank(volume, 32)\* (1 - Ts_Rank(((close + high) - low), 16)))\* (1 - Ts_Rank(returns, 32)))
    """
    enddate = to_date_str(enddate)
    func_name = sys._getframe().f_code.co_name
    return JQDataClient.instance().get_alpha_101(**locals())


@assert_auth
@hashable_lru(maxsize=3)
def alpha_036(enddate, index='all'):
    """
    Inputs:
        enddate: 必选参数，计算哪一天的因子  
        index: 默认参数，股票指数，默认为所有股票'all' 
    Outputs:
        Series：index 为成分股代码，values为对应的因子值
    公式:
        (((((2.21\* rank(correlation((close - open), delay(volume, 1), 15))) + (0.7\* rank((open - close)))) + (0.73\* rank(Ts_Rank(delay((-1\* returns), 6), 5)))) + rank(abs(correlation(vwap, adv20, 6)))) + (0.6\* rank((((sum(close, 200) / 200) - open)\* (close - open)))))
    """
    enddate = to_date_str(enddate)
    func_name = sys._getframe().f_code.co_name
    return JQDataClient.instance().get_alpha_101(**locals())


@assert_auth
@hashable_lru(maxsize=3)
def alpha_037(enddate, index='all'):
    """
    Inputs:
        enddate: 必选参数，计算哪一天的因子  
        index: 默认参数，股票指数，默认为所有股票'all' 
    Outputs:
        Series：index 为成分股代码，values为对应的因子值
    公式:
        (rank(correlation(delay((open - close), 1), close, 200)) + rank((open - close)))
    """
    enddate = to_date_str(enddate)
    func_name = sys._getframe().f_code.co_name
    return JQDataClient.instance().get_alpha_101(**locals())


@assert_auth
@hashable_lru(maxsize=3)
def alpha_038(enddate, index='all'):
    """
    Inputs:
        enddate: 必选参数，计算哪一天的因子  
        index: 默认参数，股票指数，默认为所有股票'all' 
    Outputs:
        Series：index 为成分股代码，values为对应的因子值
    公式:
        ((-1\* rank(Ts_Rank(close, 10)))\* rank((close / open)))
    """
    enddate = to_date_str(enddate)
    func_name = sys._getframe().f_code.co_name
    return JQDataClient.instance().get_alpha_101(**locals())


@assert_auth
@hashable_lru(maxsize=3)
def alpha_039(enddate, index='all'):
    """
    Inputs:
        enddate: 必选参数，计算哪一天的因子  
        index: 默认参数，股票指数，默认为所有股票'all' 
    Outputs:
        Series：index 为成分股代码，values为对应的因子值
    公式:
        ((-1\* rank((delta(close, 7)\* (1 - rank(decay_linear((volume / adv20), 9))))))\* (1 + rank(sum(returns, 250))))
    """
    enddate = to_date_str(enddate)
    func_name = sys._getframe().f_code.co_name
    return JQDataClient.instance().get_alpha_101(**locals())


@assert_auth
@hashable_lru(maxsize=3)
def alpha_040(enddate, index='all'):
    """
    Inputs:
        enddate: 必选参数，计算哪一天的因子  
        index: 默认参数，股票指数，默认为所有股票'all' 
    Outputs:
        Series：index 为成分股代码，values为对应的因子值
    公式:
        ((-1\* rank(stddev(high, 10)))\* correlation(high, volume, 10))
    """
    enddate = to_date_str(enddate)
    func_name = sys._getframe().f_code.co_name
    return JQDataClient.instance().get_alpha_101(**locals())


@assert_auth
@hashable_lru(maxsize=3)
def alpha_041(enddate, index='all'):
    """
    Inputs:
        enddate: 必选参数，计算哪一天的因子  
        index: 默认参数，股票指数，默认为所有股票'all' 
    Outputs:
        Series：index 为成分股代码，values为对应的因子值
    公式:
        (((high\* low)^0.5) - vwap)
    """
    enddate = to_date_str(enddate)
    func_name = sys._getframe().f_code.co_name
    return JQDataClient.instance().get_alpha_101(**locals())


@assert_auth
@hashable_lru(maxsize=3)
def alpha_042(enddate, index='all'):
    """
    Inputs:
        enddate: 必选参数，计算哪一天的因子  
        index: 默认参数，股票指数，默认为所有股票'all' 
    Outputs:
        Series：index 为成分股代码，values为对应的因子值
    公式:
        (rank((vwap - close)) / rank((vwap + close)))
    """
    enddate = to_date_str(enddate)
    func_name = sys._getframe().f_code.co_name
    return JQDataClient.instance().get_alpha_101(**locals())


@assert_auth
@hashable_lru(maxsize=3)
def alpha_043(enddate, index='all'):
    """
    Inputs:
        enddate: 必选参数，计算哪一天的因子  
        index: 默认参数，股票指数，默认为所有股票'all' 
    Outputs:
        Series：index 为成分股代码，values为对应的因子值
    公式:
        (ts_rank((volume / adv20), 20)\* ts_rank((-1\* delta(close, 7)), 8))
    """
    enddate = to_date_str(enddate)
    func_name = sys._getframe().f_code.co_name
    return JQDataClient.instance().get_alpha_101(**locals())


@assert_auth
@hashable_lru(maxsize=3)
def alpha_044(enddate, index='all'):
    """
    Inputs:
        enddate: 必选参数，计算哪一天的因子  
        index: 默认参数，股票指数，默认为所有股票'all' 
    Outputs:
        Series：index 为成分股代码，values为对应的因子值
    公式:
        (-1\* correlation(high, rank(volume), 5))
    """
    enddate = to_date_str(enddate)
    func_name = sys._getframe().f_code.co_name
    return JQDataClient.instance().get_alpha_101(**locals())


@assert_auth
@hashable_lru(maxsize=3)
def alpha_045(enddate, index='all'):
    """
    Inputs:
        enddate: 必选参数，计算哪一天的因子  
        index: 默认参数，股票指数，默认为所有股票'all' 
    Outputs:
        Series：index 为成分股代码，values为对应的因子值
    公式:
        (-1\* ((rank((sum(delay(close, 5), 20) / 20))\* correlation(close, volume, 2))\* rank(correlation(sum(close, 5), sum(close, 20), 2))))
    """
    enddate = to_date_str(enddate)
    func_name = sys._getframe().f_code.co_name
    return JQDataClient.instance().get_alpha_101(**locals())


@assert_auth
@hashable_lru(maxsize=3)
def alpha_046(enddate, index='all'):
    """
    Inputs:
        enddate: 必选参数，计算哪一天的因子  
        index: 默认参数，股票指数，默认为所有股票'all' 
    Outputs:
        eries：index为成分股代码，values为1或-1或对应因子值，取决于满不满足某些因子中的条件
    公式:
        ((0.25 < (((delay(close, 20) - delay(close, 10)) / 10) - ((delay(close, 10) - close) / 10))) ? (-1\* 1) : (((((delay(close, 20) - delay(close, 10)) / 10) - ((delay(close, 10) - close) / 10)) < 0) ? 1 : ((-1\* 1)\* (close - delay(close, 1)))))
    """
    enddate = to_date_str(enddate)
    func_name = sys._getframe().f_code.co_name
    return JQDataClient.instance().get_alpha_101(**locals())


@assert_auth
@hashable_lru(maxsize=3)
def alpha_047(enddate, index='all'):
    """
    Inputs:
        enddate: 必选参数，计算哪一天的因子  
        index: 默认参数，股票指数，默认为所有股票'all' 
    Outputs:
        Series：index 为成分股代码，values为对应的因子值
    公式:
        ((((rank((1 / close))\* volume) / adv20)\* ((high\* rank((high - close))) / (sum(high, 5) / 5))) - rank((vwap - delay(vwap, 5))))
    """
    enddate = to_date_str(enddate)
    func_name = sys._getframe().f_code.co_name
    return JQDataClient.instance().get_alpha_101(**locals())


@assert_auth
@hashable_lru(maxsize=3)
def alpha_048(enddate, index='all'):
    """
    未实现
    公式:
        (indneutralize(((correlation(delta(close, 1), delta(delay(close, 1), 1), 250)\* delta(close, 1)) / close), IndClass.subindustry) / sum(((delta(close, 1) / delay(close, 1))^2), 250))
    """
    raise Exception("该因子未实现")


@assert_auth
@hashable_lru(maxsize=3)
def alpha_049(enddate, index='all'):
    """
    Inputs:
        enddate: 必选参数，计算哪一天的因子  
        index: 默认参数，股票指数，默认为所有股票'all' 
    Outputs:
        eries：index为成分股代码，values为对应因子值或1，当满足条件时为1
    公式:
        (((((delay(close, 20) - delay(close, 10)) / 10) - ((delay(close, 10) - close) / 10)) < (-1\* 0.1)) ? 1 : ((-1\* 1)\* (close - delay(close, 1))))
    """
    enddate = to_date_str(enddate)
    func_name = sys._getframe().f_code.co_name
    return JQDataClient.instance().get_alpha_101(**locals())


@assert_auth
@hashable_lru(maxsize=3)
def alpha_050(enddate, index='all'):
    """
    Inputs:
        enddate: 必选参数，计算哪一天的因子  
        index: 默认参数，股票指数，默认为所有股票'all' 
    Outputs:
        Series：index 为成分股代码，values为对应的因子值
    公式:
        (-1\* ts_max(rank(correlation(rank(volume), rank(vwap), 5)), 5))
    """
    enddate = to_date_str(enddate)
    func_name = sys._getframe().f_code.co_name
    return JQDataClient.instance().get_alpha_101(**locals())


@assert_auth
@hashable_lru(maxsize=3)
def alpha_051(enddate, index='all'):
    """
    Inputs:
        enddate: 必选参数，计算哪一天的因子  
        index: 默认参数，股票指数，默认为所有股票'all' 
    Outputs:
        eries：index为成分股代码，values为对应因子值或1，当满足条件时为1
    公式:
        (((((delay(close, 20) - delay(close, 10)) / 10) - ((delay(close, 10) - close) / 10)) < (-1\* 0.05)) ? 1 : ((-1\* 1)\* (close - delay(close, 1))))
    """
    enddate = to_date_str(enddate)
    func_name = sys._getframe().f_code.co_name
    return JQDataClient.instance().get_alpha_101(**locals())


@assert_auth
@hashable_lru(maxsize=3)
def alpha_052(enddate, index='all'):
    """
    Inputs:
        enddate: 必选参数，计算哪一天的因子  
        index: 默认参数，股票指数，默认为所有股票'all' 
    Outputs:
        Series：index 为成分股代码，values为对应的因子值
    公式:
        ((((-1\* ts_min(low, 5)) + delay(ts_min(low, 5), 5))\* rank(((sum(returns, 240) - sum(returns, 20)) / 220)))\* ts_rank(volume, 5))
    """
    enddate = to_date_str(enddate)
    func_name = sys._getframe().f_code.co_name
    return JQDataClient.instance().get_alpha_101(**locals())


@assert_auth
@hashable_lru(maxsize=3)
def alpha_053(enddate, index='all'):
    """
    Inputs:
        enddate: 必选参数，计算哪一天的因子  
        index: 默认参数，股票指数，默认为所有股票'all' 
    Outputs:
        Series：index 为成分股代码，values为对应的因子值
    公式:
        (-1\* delta((((close - low) - (high - close)) / (close - low)), 9))
    """
    enddate = to_date_str(enddate)
    func_name = sys._getframe().f_code.co_name
    return JQDataClient.instance().get_alpha_101(**locals())


@assert_auth
@hashable_lru(maxsize=3)
def alpha_054(enddate, index='all'):
    """
    Inputs:
        enddate: 必选参数，计算哪一天的因子  
        index: 默认参数，股票指数，默认为所有股票'all' 
    Outputs:
        Series：index 为成分股代码，values为对应的因子值
    公式:
        ((-1\* ((low - close)\* (open^5))) / ((low - high)\* (close^5)))
    """
    enddate = to_date_str(enddate)
    func_name = sys._getframe().f_code.co_name
    return JQDataClient.instance().get_alpha_101(**locals())


@assert_auth
@hashable_lru(maxsize=3)
def alpha_055(enddate, index='all'):
    """
    Inputs:
        enddate: 必选参数，计算哪一天的因子  
        index: 默认参数，股票指数，默认为所有股票'all' 
    Outputs:
        Series：index 为成分股代码，values为对应的因子值
    公式:
        (-1\* correlation(rank(((close - ts_min(low, 12)) / (ts_max(high, 12) - ts_min(low, 12)))), rank(volume), 6))
    """
    enddate = to_date_str(enddate)
    func_name = sys._getframe().f_code.co_name
    return JQDataClient.instance().get_alpha_101(**locals())


@assert_auth
@hashable_lru(maxsize=3)
def alpha_056(enddate, index='all'):
    """
    Inputs:
        enddate: 必选参数，计算哪一天的因子  
        index: 默认参数，股票指数，默认为所有股票'all' 
    Outputs:
        Series：index 为成分股代码，values为对应的因子值
    公式:
        (0 - (1\* (rank((sum(returns, 10) / sum(sum(returns, 2), 3)))\* rank((returns\* cap)))))
    """
    enddate = to_date_str(enddate)
    func_name = sys._getframe().f_code.co_name
    return JQDataClient.instance().get_alpha_101(**locals())


@assert_auth
@hashable_lru(maxsize=3)
def alpha_057(enddate, index='all'):
    """
    Inputs:
        enddate: 必选参数，计算哪一天的因子  
        index: 默认参数，股票指数，默认为所有股票'all' 
    Outputs:
        Series：index 为成分股代码，values为对应的因子值
    公式:
        (0 - (1\* ((close - vwap) / decay_linear(rank(ts_argmax(close, 30)), 2))))
    """
    enddate = to_date_str(enddate)
    func_name = sys._getframe().f_code.co_name
    return JQDataClient.instance().get_alpha_101(**locals())


@assert_auth
@hashable_lru(maxsize=3)
def alpha_058(enddate, index='all'):
    """
    未实现
    公式:
        (-1\* Ts_Rank(decay_linear(correlation(IndNeutralize(vwap, IndClass.sector), volume, 3.92795), 7.89291), 5.50322))
    """
    raise Exception("该因子未实现")


@assert_auth
@hashable_lru(maxsize=3)
def alpha_059(enddate, index='all'):
    """
    未实现
    公式:
        (-1\* Ts_Rank(decay_linear(correlation(IndNeutralize(((vwap\* 0.728317) + (vwap\* (1 - 0.728317))), IndClass.industry), volume, 4.25197), 16.2289), 8.19648))
    """
    raise Exception("该因子未实现")


@assert_auth
@hashable_lru(maxsize=3)
def alpha_060(enddate, index='all'):
    """
    Inputs:
        enddate: 必选参数，计算哪一天的因子  
        index: 默认参数，股票指数，默认为所有股票'all' 
    Outputs:
        Series：index 为成分股代码，values为对应的因子值
    公式:
        (0 - (1\* ((2\* scale(rank(((((close - low) - (high - close)) / (high - low))\* volume)))) - scale(rank(ts_argmax(close, 10))))))
    """
    enddate = to_date_str(enddate)
    func_name = sys._getframe().f_code.co_name
    return JQDataClient.instance().get_alpha_101(**locals())


@assert_auth
@hashable_lru(maxsize=3)
def alpha_061(enddate, index='all'):
    """
    Inputs:
        enddate: 必选参数，计算哪一天的因子  
        index: 默认参数，股票指数，默认为所有股票'all' 
    Outputs:
        eries：index为成分股代码，values为1或-1，当满足条件时为1，否则为-1
    公式:
        (rank((vwap - ts_min(vwap, 16.1219))) < rank(correlation(vwap, adv180, 17.9282)))
    """
    enddate = to_date_str(enddate)
    func_name = sys._getframe().f_code.co_name
    return JQDataClient.instance().get_alpha_101(**locals())


@assert_auth
@hashable_lru(maxsize=3)
def alpha_062(enddate, index='all'):
    """
    Inputs:
        enddate: 必选参数，计算哪一天的因子  
        index: 默认参数，股票指数，默认为所有股票'all' 
    Outputs:
        eries：index为成分股代码，values为1或-1，当满足条件时为-1，否则为1
    公式:
        ((rank(correlation(vwap, sum(adv20, 22.4101), 9.91009)) < rank(((rank(open) + rank(open)) < (rank(((high + low) / 2)) + rank(high)))))\*-1)
    """
    enddate = to_date_str(enddate)
    func_name = sys._getframe().f_code.co_name
    return JQDataClient.instance().get_alpha_101(**locals())


@assert_auth
@hashable_lru(maxsize=3)
def alpha_063(enddate, index='all'):
    """
    未实现
    公式:
        ((rank(decay_linear(delta(IndNeutralize(close, IndClass.industry), 2.25164), 8.22237)) - rank(decay_linear(correlation(((vwap\* 0.318108) + (open\* (1 - 0.318108))), sum(adv180, 37.2467), 13.557), 12.2883)))\* -1)
    """
    raise Exception("该因子未实现")


@assert_auth
@hashable_lru(maxsize=3)
def alpha_064(enddate, index='all'):
    """
    Inputs:
        enddate: 必选参数，计算哪一天的因子  
        index: 默认参数，股票指数，默认为所有股票'all' 
    Outputs:
        eries：index为成分股代码，values为1或-1，当满足条件时为-1，否则为1
    公式:
        ((rank(correlation(sum(((open\* 0.178404) + (low\* (1 - 0.178404))), 12.7054), sum(adv120, 12.7054), 16.6208)) < rank(delta(((((high + low) / 2)\* 0.178404) + (vwap\* (1 - 0.178404))), 3.69741)))\* -1)
    """
    enddate = to_date_str(enddate)
    func_name = sys._getframe().f_code.co_name
    return JQDataClient.instance().get_alpha_101(**locals())


@assert_auth
@hashable_lru(maxsize=3)
def alpha_065(enddate, index='all'):
    """
    Inputs:
        enddate: 必选参数，计算哪一天的因子  
        index: 默认参数，股票指数，默认为所有股票'all' 
    Outputs:
        eries：index为成分股代码，values为1或-1，当满足条件时为-1，否则为1
    公式:
         ((rank(correlation(((open\* 0.00817205) + (vwap\* (1 - 0.00817205))), sum(adv60, 8.6911), 6.40374)) < rank((open - ts_min(open, 13.635))))\* -1)
    """
    enddate = to_date_str(enddate)
    func_name = sys._getframe().f_code.co_name
    return JQDataClient.instance().get_alpha_101(**locals())


@assert_auth
@hashable_lru(maxsize=3)
def alpha_066(enddate, index='all'):
    """
    Inputs:
        enddate: 必选参数，计算哪一天的因子  
        index: 默认参数，股票指数，默认为所有股票'all' 
    Outputs:
        Series：index 为成分股代码，values为对应的因子值
    公式:
        ((rank(decay_linear(delta(vwap, 3.51013), 7.23052)) + Ts_Rank(decay_linear(((((low\* 0.96633) + (low\* (1 - 0.96633))) - vwap) / (open - ((high + low) / 2))), 11.4157), 6.72611))\* -1)
    """
    enddate = to_date_str(enddate)
    func_name = sys._getframe().f_code.co_name
    return JQDataClient.instance().get_alpha_101(**locals())


@assert_auth
@hashable_lru(maxsize=3)
def alpha_067(enddate, index='all'):
    """
    未实现
    公式:
        ((rank((high - ts_min(high, 2.14593)))^rank(correlation(IndNeutralize(vwap, IndClass.sector), IndNeutralize(adv20, IndClass.subindustry), 6.02936)))\* -1)
    """
    raise Exception("该因子未实现")


@assert_auth
@hashable_lru(maxsize=3)
def alpha_068(enddate, index='all'):
    """
    Inputs:
        enddate: 必选参数，计算哪一天的因子  
        index: 默认参数，股票指数，默认为所有股票'all' 
    Outputs:
        eries：index为成分股代码，values为1或-1，当满足条件时为-1，否则为1
    公式:
        ((Ts_Rank(correlation(rank(high), rank(adv15), 8.91644), 13.9333) < rank(delta(((close\* 0.518371) + (low\* (1 - 0.518371))), 1.06157)))\* -1)
    """
    enddate = to_date_str(enddate)
    func_name = sys._getframe().f_code.co_name
    return JQDataClient.instance().get_alpha_101(**locals())


@assert_auth
@hashable_lru(maxsize=3)
def alpha_069(enddate, index='all'):
    """
    未实现
    公式:
        ((rank(ts_max(delta(IndNeutralize(vwap, IndClass.industry), 2.72412), 4.79344))^Ts_Rank(correlation(((close\* 0.490655) + (vwap\* (1 - 0.490655))), adv20, 4.92416), 9.0615))\* -1)
    """
    raise Exception("该因子未实现")


@assert_auth
@hashable_lru(maxsize=3)
def alpha_070(enddate, index='all'):
    """
    未实现
    公式:
        ((rank(delta(vwap, 1.29456))^Ts_Rank(correlation(IndNeutralize(close, IndClass.industry), adv50, 17.8256), 17.9171))\* -1)
    """
    raise Exception("该因子未实现")


@assert_auth
@hashable_lru(maxsize=3)
def alpha_071(enddate, index='all'):
    """
    Inputs:
        enddate: 必选参数，计算哪一天的因子  
        index: 默认参数，股票指数，默认为所有股票'all' 
    Outputs:
        Series：index 为成分股代码，values为对应的因子值
    公式:
        max(Ts_Rank(decay_linear(correlation(Ts_Rank(close, 3.43976), Ts_Rank(adv180, 12.0647), 18.0175), 4.20501), 15.6948), Ts_Rank(decay_linear((rank(((low + open) - (vwap + vwap)))^2), 16.4662), 4.4388))
    """
    enddate = to_date_str(enddate)
    func_name = sys._getframe().f_code.co_name
    return JQDataClient.instance().get_alpha_101(**locals())


@assert_auth
@hashable_lru(maxsize=3)
def alpha_072(enddate, index='all'):
    """
    Inputs:
        enddate: 必选参数，计算哪一天的因子  
        index: 默认参数，股票指数，默认为所有股票'all' 
    Outputs:
        Series：index 为成分股代码，values为对应的因子值
    公式:
        (rank(decay_linear(correlation(((high + low) / 2), adv40, 8.93345), 10.1519)) / rank(decay_linear(correlation(Ts_Rank(vwap, 3.72469), Ts_Rank(volume, 18.5188), 6.86671), 2.95011)))
    """
    enddate = to_date_str(enddate)
    func_name = sys._getframe().f_code.co_name
    return JQDataClient.instance().get_alpha_101(**locals())


@assert_auth
@hashable_lru(maxsize=3)
def alpha_073(enddate, index='all'):
    """
    Inputs:
        enddate: 必选参数，计算哪一天的因子  
        index: 默认参数，股票指数，默认为所有股票'all' 
    Outputs:
        Series：index 为成分股代码，values为对应的因子值
    公式:
        (max(rank(decay_linear(delta(vwap, 4.72775), 2.91864)), Ts_Rank(decay_linear(((delta(((open\* 0.147155) + (low\* (1 - 0.147155))), 2.03608) / ((open\* 0.147155) + (low\* (1 - 0.147155))))\* -1), 3.33829), 16.7411))\* -1)
    """
    enddate = to_date_str(enddate)
    func_name = sys._getframe().f_code.co_name
    return JQDataClient.instance().get_alpha_101(**locals())


@assert_auth
@hashable_lru(maxsize=3)
def alpha_074(enddate, index='all'):
    """
    Inputs:
        enddate: 必选参数，计算哪一天的因子  
        index: 默认参数，股票指数，默认为所有股票'all' 
    Outputs:
        eries：index为成分股代码，values为1或-1，当满足条件时为-1，否则为1
    公式:
        ((rank(correlation(close, sum(adv30, 37.4843), 15.1365)) < rank(correlation(rank(((high\* 0.0261661) + (vwap\* (1 - 0.0261661)))), rank(volume), 11.4791)))\* -1)
    """
    enddate = to_date_str(enddate)
    func_name = sys._getframe().f_code.co_name
    return JQDataClient.instance().get_alpha_101(**locals())


@assert_auth
@hashable_lru(maxsize=3)
def alpha_075(enddate, index='all'):
    """
    Inputs:
        enddate: 必选参数，计算哪一天的因子  
        index: 默认参数，股票指数，默认为所有股票'all' 
    Outputs:
        eries：index为成分股代码，values为1或-1，当满足条件时为1，否则为-1 
    公式:
        (rank(correlation(vwap, volume, 4.24304)) < rank(correlation(rank(low), rank(adv50), 12.4413)))
    """
    enddate = to_date_str(enddate)
    func_name = sys._getframe().f_code.co_name
    return JQDataClient.instance().get_alpha_101(**locals())


@assert_auth
@hashable_lru(maxsize=3)
def alpha_076(enddate, index='all'):
    """
    未实现
    公式:
        (max(rank(decay_linear(delta(vwap, 1.24383), 11.8259)), Ts_Rank(decay_linear(Ts_Rank(correlation(IndNeutralize(low, IndClass.sector), adv81, 8.14941), 19.569), 17.1543), 19.383))\* -1)
    """
    raise Exception("该因子未实现")


@assert_auth
@hashable_lru(maxsize=3)
def alpha_077(enddate, index='all'):
    """
    Inputs:
        enddate: 必选参数，计算哪一天的因子  
        index: 默认参数，股票指数，默认为所有股票'all' 
    Outputs:
        Series：index 为成分股代码，values为对应的因子值
    公式:
        min(rank(decay_linear(((((high + low) / 2) + high) - (vwap + high)), 20.0451)), rank(decay_linear(correlation(((high + low) / 2), adv40, 3.1614), 5.64125)))
    """
    enddate = to_date_str(enddate)
    func_name = sys._getframe().f_code.co_name
    return JQDataClient.instance().get_alpha_101(**locals())


@assert_auth
@hashable_lru(maxsize=3)
def alpha_078(enddate, index='all'):
    """
    Inputs:
        enddate: 必选参数，计算哪一天的因子  
        index: 默认参数，股票指数，默认为所有股票'all' 
    Outputs:
        Series：index 为成分股代码，values为对应的因子值
    公式:
        (rank(correlation(sum(((low\* 0.352233) + (vwap\* (1 - 0.352233))), 19.7428), sum(adv40, 19.7428), 6.83313))^rank(correlation(rank(vwap), rank(volume), 5.77492)))
    """
    enddate = to_date_str(enddate)
    func_name = sys._getframe().f_code.co_name
    return JQDataClient.instance().get_alpha_101(**locals())


@assert_auth
@hashable_lru(maxsize=3)
def alpha_079(enddate, index='all'):
    """
    未实现
    公式:
        (rank(delta(IndNeutralize(((close\* 0.60733) + (open\* (1 - 0.60733))), IndClass.sector), 1.23438)) < rank(correlation(Ts_Rank(vwap, 3.60973), Ts_Rank(adv150, 9.18637), 14.6644)))
    """
    raise Exception("该因子未实现")


@assert_auth
@hashable_lru(maxsize=3)
def alpha_080(enddate, index='all'):
    """
    未实现
    公式:
        ((rank(Sign(delta(IndNeutralize(((open\* 0.868128) + (high\* (1 - 0.868128))), IndClass.industry), 4.04545)))^Ts_Rank(correlation(high, adv10, 5.11456), 5.53756))\* -1)
    """
    raise Exception("该因子未实现")


@assert_auth
@hashable_lru(maxsize=3)
def alpha_081(enddate, index='all'):
    """
    未实现
    公式:
        ((rank(Log(product(rank((rank(correlation(vwap, sum(adv10, 49.6054), 8.47743))^4)), 14.9655))) < rank(correlation(rank(vwap), rank(volume), 5.07914)))\* -1)
    """
    raise Exception("该因子未实现")


@assert_auth
@hashable_lru(maxsize=3)
def alpha_082(enddate, index='all'):
    """
    未实现
    公式:
        (min(rank(decay_linear(delta(open, 1.46063), 14.8717)), Ts_Rank(decay_linear(correlation(IndNeutralize(volume, IndClass.sector), ((open\* 0.634196) + (open\* (1 - 0.634196))), 17.4842), 6.92131), 13.4283))\* -1)
    """
    raise Exception("该因子未实现")


@assert_auth
@hashable_lru(maxsize=3)
def alpha_083(enddate, index='all'):
    """
    Inputs:
        enddate: 必选参数，计算哪一天的因子  
        index: 默认参数，股票指数，默认为所有股票'all' 
    Outputs:
        Series：index 为成分股代码，values为对应的因子值
    公式:
        ((rank(delay(((high - low) / (sum(close, 5) / 5)), 2))\* rank(rank(volume))) / (((high - low) / (sum(close, 5) / 5)) / (vwap - close)))
    """
    enddate = to_date_str(enddate)
    func_name = sys._getframe().f_code.co_name
    return JQDataClient.instance().get_alpha_101(**locals())


@assert_auth
@hashable_lru(maxsize=3)
def alpha_084(enddate, index='all'):
    """
    Inputs:
        enddate: 必选参数，计算哪一天的因子  
        index: 默认参数，股票指数，默认为所有股票'all' 
    Outputs:
        Series：index 为成分股代码，values为对应的因子值
    公式:
        SignedPower(Ts_Rank((vwap - ts_max(vwap, 15.3217)), 20.7127), delta(close, 4.96796))
    """
    enddate = to_date_str(enddate)
    func_name = sys._getframe().f_code.co_name
    return JQDataClient.instance().get_alpha_101(**locals())


@assert_auth
@hashable_lru(maxsize=3)
def alpha_085(enddate, index='all'):
    """
    Inputs:
        enddate: 必选参数，计算哪一天的因子  
        index: 默认参数，股票指数，默认为所有股票'all' 
    Outputs:
        Series：index 为成分股代码，values为对应的因子值
    公式:
        (rank(correlation(((high\* 0.876703) + (close\* (1 - 0.876703))), adv30, 9.61331))^rank(correlation(Ts_Rank(((high + low) / 2), 3.70596), Ts_Rank(volume, 10.1595), 7.11408)))
    """
    enddate = to_date_str(enddate)
    func_name = sys._getframe().f_code.co_name
    return JQDataClient.instance().get_alpha_101(**locals())


@assert_auth
@hashable_lru(maxsize=3)
def alpha_086(enddate, index='all'):
    """
    Inputs:
        enddate: 必选参数，计算哪一天的因子  
        index: 默认参数，股票指数，默认为所有股票'all' 
    Outputs:
        eries：index为成分股代码，values为1或-1，当满足条件时为-1，否则为1
    公式:
        ((Ts_Rank(correlation(close, sum(adv20, 14.7444), 6.00049), 20.4195) < rank(((open + close) - (vwap + open))))\* -1)
    """
    enddate = to_date_str(enddate)
    func_name = sys._getframe().f_code.co_name
    return JQDataClient.instance().get_alpha_101(**locals())


@assert_auth
@hashable_lru(maxsize=3)
def alpha_087(enddate, index='all'):
    """
    未实现
    公式:
        (max(rank(decay_linear(delta(((close\* 0.369701) + (vwap\* (1 - 0.369701))), 1.91233), 2.65461)), Ts_Rank(decay_linear(abs(correlation(IndNeutralize(adv81, IndClass.industry), close, 13.4132)), 4.89768), 14.4535))\* -1)
    """
    raise Exception("该因子未实现")


@assert_auth
@hashable_lru(maxsize=3)
def alpha_088(enddate, index='all'):
    """
    Inputs:
        enddate: 必选参数，计算哪一天的因子  
        index: 默认参数，股票指数，默认为所有股票'all' 
    Outputs:
        Series：index 为成分股代码，values为对应的因子值
    公式:
        min(rank(decay_linear(((rank(open) + rank(low)) - (rank(high) + rank(close))), 8.06882)), Ts_Rank(decay_linear(correlation(Ts_Rank(close, 8.44728), Ts_Rank(adv60, 20.6966), 8.01266), 6.65053), 2.61957))
    """
    enddate = to_date_str(enddate)
    func_name = sys._getframe().f_code.co_name
    return JQDataClient.instance().get_alpha_101(**locals())


@assert_auth
@hashable_lru(maxsize=3)
def alpha_089(enddate, index='all'):
    """
    未实现
    公式:
        (Ts_Rank(decay_linear(correlation(((low\* 0.967285) + (low\* (1 - 0.967285))), adv10, 6.94279), 5.51607), 3.79744) - Ts_Rank(decay_linear(delta(IndNeutralize(vwap, IndClass.industry), 3.48158), 10.1466), 15.3012))
    """
    raise Exception("该因子未实现")



@assert_auth
@hashable_lru(maxsize=3)
def alpha_090(enddate, index='all'):
    """
    未实现
    公式:
        ((rank((close - ts_max(close, 4.66719)))^Ts_Rank(correlation(IndNeutralize(adv40, IndClass.subindustry), low, 5.38375), 3.21856))\* -1)
    """
    raise Exception("该因子未实现")


@assert_auth
@hashable_lru(maxsize=3)
def alpha_091(enddate, index='all'):
    """
    未实现
    公式:
        ((Ts_Rank(decay_linear(decay_linear(correlation(IndNeutralize(close, IndClass.industry), volume, 9.74928), 16.398), 3.83219), 4.8667) - rank(decay_linear(correlation(vwap, adv30, 4.01303), 2.6809)))\* -1)
    """
    raise Exception("该因子未实现")


@assert_auth
@hashable_lru(maxsize=3)
def alpha_092(enddate, index='all'):
    """
    Inputs:
        enddate: 必选参数，计算哪一天的因子  
        index: 默认参数，股票指数，默认为所有股票'all' 
    Outputs:
        Series：index 为成分股代码，values为对应的因子值
    公式:
        min(Ts_Rank(decay_linear(((((high + low) / 2) + close) < (low + open)), 14.7221), 18.8683), Ts_Rank(decay_linear(correlation(rank(low), rank(adv30), 7.58555), 6.94024), 6.80584))
    """
    enddate = to_date_str(enddate)
    func_name = sys._getframe().f_code.co_name
    return JQDataClient.instance().get_alpha_101(**locals())


@assert_auth
@hashable_lru(maxsize=3)
def alpha_093(enddate, index='all'):
    """
    未实现
    公式:
        (Ts_Rank(decay_linear(correlation(IndNeutralize(vwap, IndClass.industry), adv81, 17.4193), 19.848), 7.54455) / rank(decay_linear(delta(((close\* 0.524434) + (vwap\* (1 - 0.524434))), 2.77377), 16.2664)))
    """
    raise Exception("该因子未实现")


@assert_auth
@hashable_lru(maxsize=3)
def alpha_094(enddate, index='all'):
    """
    Inputs:
        enddate: 必选参数，计算哪一天的因子  
        index: 默认参数，股票指数，默认为所有股票'all' 
    Outputs:
        Series：index 为成分股代码，values为对应的因子值
    公式:
        ((rank((vwap - ts_min(vwap, 11.5783)))^Ts_Rank(correlation(Ts_Rank(vwap, 19.6462), Ts_Rank(adv60, 4.02992), 18.0926), 2.70756))\* -1)
    """
    enddate = to_date_str(enddate)
    func_name = sys._getframe().f_code.co_name
    return JQDataClient.instance().get_alpha_101(**locals())


@assert_auth
@hashable_lru(maxsize=3)
def alpha_095(enddate, index='all'):
    """
    Inputs:
        enddate: 必选参数，计算哪一天的因子  
        index: 默认参数，股票指数，默认为所有股票'all' 
    Outputs:
        eries：index为成分股代码，values为1或-1，当满足条件时为1，否则为-1
    公式:
        (rank((open - ts_min(open, 12.4105))) < Ts_Rank((rank(correlation(sum(((high + low) / 2), 19.1351), sum(adv40, 19.1351), 12.8742))^5), 11.7584))
    """
    enddate = to_date_str(enddate)
    func_name = sys._getframe().f_code.co_name
    return JQDataClient.instance().get_alpha_101(**locals())


@assert_auth
@hashable_lru(maxsize=3)
def alpha_096(enddate, index='all'):
    """
    Inputs:
        enddate: 必选参数，计算哪一天的因子  
        index: 默认参数，股票指数，默认为所有股票'all' 
    Outputs:
        Series：index 为成分股代码，values为对应的因子值
    公式:
        (max(Ts_Rank(decay_linear(correlation(rank(vwap), rank(volume), 3.83878), 4.16783), 8.38151), Ts_Rank(decay_linear(Ts_ArgMax(correlation(Ts_Rank(close, 7.45404), Ts_Rank(adv60, 4.13242), 3.65459), 12.6556), 14.0365), 13.4143))\* -1)
    """
    enddate = to_date_str(enddate)
    func_name = sys._getframe().f_code.co_name
    return JQDataClient.instance().get_alpha_101(**locals())


@assert_auth
@hashable_lru(maxsize=3)
def alpha_097(enddate, index='all'):
    """
    未实现
    公式: 
        ((rank(decay_linear(delta(IndNeutralize(((low\* 0.721001) + (vwap\* (1 - 0.721001))), IndClass.industry), 3.3705), 20.4523)) - Ts_Rank(decay_linear(Ts_Rank(correlation(Ts_Rank(low, 7.87871), Ts_Rank(adv60, 17.255), 4.97547), 18.5925), 15.7152), 6.71659))\* -1)
    """
    raise Exception("该因子未实现")


@assert_auth
@hashable_lru(maxsize=3)
def alpha_098(enddate, index='all'):
    """
    Inputs:
        enddate: 必选参数，计算哪一天的因子  
        index: 默认参数，股票指数，默认为所有股票'all' 
    Outputs:
        Series：index 为成分股代码，values为对应的因子值
    公式:
        (rank(decay_linear(correlation(vwap, sum(adv5, 26.4719), 4.58418), 7.18088)) - rank(decay_linear(Ts_Rank(Ts_ArgMin(correlation(rank(open), rank(adv15), 20.8187), 8.62571), 6.95668), 8.07206)))
    """
    enddate = to_date_str(enddate)
    func_name = sys._getframe().f_code.co_name
    return JQDataClient.instance().get_alpha_101(**locals())

@assert_auth
@hashable_lru(maxsize=3)
def alpha_099(enddate, index='all'):
    """
    Inputs:
        enddate: 必选参数，计算哪一天的因子  
        index: 默认参数，股票指数，默认为所有股票'all' 
    Outputs:
        Series: index为成分股代码，values为1或-1，当满足条件时为-1，否则为1
    公式:
        ((rank(correlation(sum(((high + low) / 2), 19.8975), sum(adv60, 19.8975), 8.8136)) < rank(correlation(low, volume, 6.28259)))\* -1)
    """
    enddate = to_date_str(enddate)
    func_name = sys._getframe().f_code.co_name
    return JQDataClient.instance().get_alpha_101(**locals())


@assert_auth
@hashable_lru(maxsize=3)
def alpha_100(enddate, index='all'):
    """
    未实现
    公式:
        (0 - (1\* (((1.5\* scale(indneutralize(indneutralize(rank(((((close - low) - (high - close)) / (high - low))\* volume)), IndClass.subindustry), IndClass.subindustry))) - scale(indneutralize((correlation(close, rank(adv20), 5) - rank(ts_argmin(close, 30))), IndClass.subindustry)))\* (volume / adv20))))
    """
    raise Exception("该因子未实现")


@assert_auth
@hashable_lru(maxsize=3)
def alpha_101(enddate, index='all'):
    """
    Inputs:
        enddate: 必选参数，计算哪一天的因子  
        index: 默认参数，股票指数，默认为所有股票'all' 
    Outputs:
        Series：index 为成分股代码，values为对应的因子值
    公式:
        ((close - open) / ((high - low) + .001))
    """
    enddate = to_date_str(enddate)
    func_name = sys._getframe().f_code.co_name
    return JQDataClient.instance().get_alpha_101(**locals())


__all__ = [k for k, v in sorted(globals().items()) if k.startswith("alpha_")]





# coding=utf-8
import sys
import os
sys.modules["ROOT_DIR"] = os.path.abspath(os.path.dirname(__file__))

from .api import *
from .finance_service import *
from . import alpha101
from . import alpha191
from . import technical_analysis
from .table import *
from .client import JQDataClient, AnalysisDNS

__version__ = "1.7.8"

def auth(username, password, host="39.107.190.114", port=7000):
    JQDataClient.set_auth_params(host=host, port=port, username=username, password=password, version=__version__)

def auth_by_token(token, host="39.107.190.114", port=7000):
    JQDataClient.set_auth_params(host=host, port=port, token=token)

def logout():
    JQDataClient.instance().logout()

def is_auth():
    if not JQDataClient.instance():
        return False
    return not JQDataClient.instance().not_auth

analysis_dns = AnalysisDNS()
analysis_dns.start()

__all__ = ["auth", "auth_by_token", "logout", "is_auth", "alpha101", "alpha191", "technical_analysis", "__version__"]
__all__.extend(api.__all__)
__all__.extend(finance_service.__all__)
__all__.extend(table.__all__)



# coding=utf-8
from functools import wraps
from io import StringIO
import requests
from .utils import *
from .client import JQDataClient

@assert_auth
def get_price(security, start_date=None, end_date=None, frequency='daily',
              fields=None, skip_paused=False, fq='pre', count=None, panel=True, fill_paused=True):
    """
    获取一支或者多只证券的行情数据

    :param security 一支证券代码或者一个证券代码的list
    :param count 与 start_date 二选一，不可同时使用.数量, 返回的结果集的行数, 即表示获取 end_date 之前几个 frequency 的数据
    :param start_date 与 count 二选一，不可同时使用. 字符串或者 datetime.datetime/datetime.date 对象, 开始时间
    :param end_date 格式同上, 结束时间, 默认是'2015-12-31', 包含此日期.
    :param frequency 单位时间长度, 几天或者几分钟, 现在支持'Xd','Xm', 'daily'(等同于'1d'), 'minute'(等同于'1m'), X是一个正整数, 分别表示X天和X分钟
    :param fields 字符串list, 默认是None(表示['open', 'close', 'high', 'low', 'volume', 'money']这几个标准字段), 支持以下属性 ['open', 'close', 'low', 'high', 'volume', 'money', 'factor', 'high_limit', 'low_limit', 'avg', 'pre_close', 'paused']
    :param skip_paused 是否跳过不交易日期(包括停牌, 未上市或者退市后的日期). 如果不跳过, 停牌时会使用停牌前的数据填充, 上市前或者退市后数据都为 nan
    :param panel: 当传入一个标的列表的时候，是否返回一个panel对象，默认为True，表示返回一哥panel对象
           注意：
               当security为一个标的列表，且panel=False的时候，会返回一个dataframe对象，
               在这个对象中额外多出code、time两个字段，分别表示该条数据对应的标的、时间
    :param fill_paused : False 表示使用NAN填充停牌的数据，True表示用close价格填充，默认True
    :return 如果是一支证券, 则返回pandas.DataFrame对象, 行索引是datetime.datetime对象, 列索引是行情字段名字; 如果是多支证券, 则返回pandas.Panel对象, 里面是很多pandas.DataFrame对象, 索引是行情字段(open/close/…), 每个pandas.DataFrame的行索引是datetime.datetime对象, 列索引是证券代号.
    """
    security = convert_security(security)
    start_date = to_date_str(start_date)
    end_date = to_date_str(end_date)
    if (not count) and (not start_date):
        start_date = "2015-01-01"
    if count and start_date:
        raise ParamsError("(start_date, count) only one param is required")
    return JQDataClient.instance().get_price(**locals())


@assert_auth
def get_price_engine(security, start_date=None, end_date=None,
                     frequency='daily', fields=None, skip_paused=False,
                     fq='pre', count=None, pre_factor_ref_date=None):
    security = convert_security(security)
    start_date = to_date_str(start_date)
    end_date = to_date_str(end_date)
    pre_factor_ref_date = to_date_str(end_date)
    return JQDataClient.instance().get_price_engine(**locals())


@assert_auth
def get_extras(info, security_list, start_date=None, end_date=None, df=True, count=None):
    """
    得到多只标的在一段时间的如下额外的数据

    :param info ['is_st', 'acc_net_value', 'unit_net_value', 'futures_sett_price', 'futures_positions'] 中的一个
    :param security_list 证券列表
    :param start_date 开始日期
    :param end_date 结束日期
    :param df 返回pandas.DataFrame对象还是一个dict
    :param count 数量, 与 start_date 二选一, 不可同时使用, 必须大于 0
    :return <df=True>:pandas.DataFrame对象, 列索引是股票代号, 行索引是datetime.datetime；<df=False>:一个dict, key是基金代号, value是numpy.ndarray
    """
    assert security_list, "security_list is required"
    start_date = to_date_str(start_date)
    end_date = to_date_str(end_date)
    security_list = convert_security(security_list)
    return JQDataClient.instance().get_extras(**locals())


@assert_auth
def get_fundamentals(query_object, date=None, statDate=None):
    """
    查询财务数据, 详细的数据字段描述在 https://www.joinquant.com/data/dict/fundamentals 中查看

    :param query_object 一个sqlalchemy.orm.query.Query对象
    :param date 查询日期, 一个字符串(格式类似’2015-10-15’)或者datetime.date/datetime.datetime对象, 可以是None, 使用默认日期
    :param statDate: 财报统计的季度或者年份, 一个字符串, 有两种格式:1.季度: 格式是: 年 + ‘q’ + 季度序号, 例如: ‘2015q1’, ‘2013q4’. 2.年份: 格式就是年份的数字, 例如: ‘2015’, ‘2016’.
    :return 返回一个 pandas.DataFrame, 每一行对应数据库返回的每一行(可能是几个表的联合查询结果的一行), 列索引是你查询的所有字段;为了防止返回数据量过大, 我们每次最多返回10000行;当相关股票上市前、退市后，财务数据返回各字段为空
    """
    if date is None and statDate is None:
        date = datetime.date.today() - datetime.timedelta(days=1)

    from .finance_service import get_fundamentals_sql
    sql = get_fundamentals_sql(query_object, date, statDate)
    if date == datetime.date.today() or date == datetime.date.today().strftime("%Y-%m-%d"):
        """ 当天的数据可能变化,不用缓存 """
        return JQDataClient.instance().get_fundamentals(sql=sql)
    return exec_fundamentals(sql)

@hashable_lru(maxsize=3)
def exec_fundamentals(sql):
    return JQDataClient.instance().get_fundamentals(sql=sql)


@assert_auth
@hashable_lru(maxsize=3)
def get_fundamentals_continuously(query_object, end_date=None, count=1, panel=True):
    """
    查询财务数据，详细的数据字段描述在 https://www.joinquant.com/data/dict/fundamentals 中查看
    :param query_object:一个sqlalchemy.orm.query.Query对象
    :param end_date:查询日期, 一个字符串(格式类似’2015-10-15’)或者datetime.date/datetime.datetime对象, 可以是None, 使用默认日期
    :param count:获取 end_date 前 count 个日期的数据
    :param panel:是否返回panel对象，默认为True，表示返回panel对象。
    :return
    :panel=true: pd.Panel, 三维分别是 field, date, security.
    :panel=False: pd.Fataframe, index是pandas默认的整数，并且始终会返回'day', 'code'这两个字段
    """
    assert count, "count is required"
    from .finance_service import fundamentals_redundant_continuously_query_to_sql
    from .calendar_service import CalendarService

    trade_days = CalendarService.get_trade_days(end_date=end_date, count=count)
    sql = fundamentals_redundant_continuously_query_to_sql(query_object, trade_days)
    sql = remove_duplicated_tables(sql)
    df = JQDataClient.instance().get_fundamentals_continuously(sql=sql)
    if panel:
        newdf = df.set_index(['day', 'code'])
        pan = newdf.to_panel()
        return pan
    else:
        try:
            df.sort(columns=['code', 'day'], inplace=True)
        except AttributeError:
            df.sort_values(by=['code', 'day'], inplace=True)
        df.reset_index(drop=True, inplace=True)
        return df


@assert_auth
@hashable_lru(maxsize=3)
def get_billboard_list(stock_list=None, start_date=None, end_date=None, count=None):
    """
    获取指定日期区间内的龙虎榜数据

    :param stock_list:一个股票代码的 list。 当值为 None 时， 返回指定日期的所有股票。
    :param start_date:开始日期
    :param end_date:结束日期
    :param count:交易日数量， 可以与 end_date 同时使用， 表示获取 end_date 前 count 个交易日的数据(含 end_date 当日)
    :return:回一个 pandas.DataFrame
    """
    start_date = to_date_str(start_date)
    end_date = to_date_str(end_date)
    stock_list = convert_security(stock_list)
    return JQDataClient.instance().get_billboard_list(**locals())


@assert_auth
def get_locked_shares(stock_list=None, start_date=None, end_date=None, forward_count=None):
    """
    获取指定日期区间内的限售解禁数据

    :param stock_list:一个股票代码的 list
    :param start_date:开始日期
    :param end_date:结束日期
    :param forward_count:交易日数量， 可以与 start_date 同时使用， 表示获取 start_date 到 forward_count 个交易日区间的数据
    :return:
    """
    start_date = to_date_str(start_date)
    end_date = to_date_str(end_date)
    stock_list = convert_security(stock_list)
    return JQDataClient.instance().get_locked_shares(**locals())


@assert_auth
@hashable_lru(maxsize=3)
def get_index_stocks(index_symbol, date=today()):
    """
    获取一个指数给定日期在平台可交易的成分股列表，请点击 https://www.joinquant.com/indexData 查看指数信息

    :param index_symbol 指数代码
    :param date: 查询日期, 一个字符串(格式类似’2015-10-15’)或者datetime.date/datetime.datetime对象, 可以是None, 使用默认日期.
    :return 股票代码的list
    """
    assert index_symbol, "index_symbol is required"
    date = to_date_str(date)
    return JQDataClient.instance().get_index_stocks(**locals())


@assert_auth
@hashable_lru(maxsize=3)
def get_industry_stocks(industry_code, date=today()):
    """
    获取在给定日期一个行业的所有股票，行业分类列表见 https://www.joinquant.com/data/dict/plateData

    :param industry_code 行业编码
    :param date 查询日期, 一个字符串(格式类似’2015-10-15’)或者datetime.date/datetime.datetime对象, 可以是None, 使用默认日期.
    :return 股票代码的list
    """
    assert industry_code, "industry_code is required"
    date = to_date_str(date)
    return JQDataClient.instance().get_industry_stocks(**locals())


@assert_auth
@hashable_lru(maxsize=3)
def get_industries(name='zjw', date=None):
    """
    按照行业分类获取行业列表
    :param name:行业代码
    :param date:获取数据的日期，默认为None
    :return:pandas.DataFrame, 各column分别为行业代码、行业名称、开始日期
    """
    assert name, "name is required"
    date = to_date_str(date)
    return JQDataClient.instance().get_industries(**locals())


@assert_auth
@hashable_lru(maxsize=3)
def get_concept_stocks(concept_code, date=today()):
    """
    获取在给定日期一个概念板块的所有股票，概念板块分类列表见 https://www.joinquant.com/data/dict/plateData

    :param concept_code 概念板块编码
    :param date: 查询日期, 一个字符串(格式类似’2015-10-15’)或者datetime.date/datetime.datetime对象, 可以是None, 使用默认日期.
    :return 股票代码的list
    """
    assert concept_code, "concept_code is required"
    date = to_date_str(date)
    return JQDataClient.instance().get_concept_stocks(**locals())


@assert_auth
@hashable_lru(maxsize=3)
def get_concepts():
    """
    获取概念板块
    :return:pandas.DataFrame, 各column分别为概念代码、概念名称、开始日期
    """
    return JQDataClient.instance().get_concepts(**locals())


@assert_auth
@hashable_lru(maxsize=3)
def get_concept(security, date):
    """
    获取股票所属概念板块。

    :param security 标的代码或标的列表
    :param date 要查询的提起, 日期字符串/date对象/datetime对象, 注意传入datetime对象时忽略日内时间
    :return:返回dict, key为标的代码, value为概念板块信息
    """
    date = to_date_str(date)
    return JQDataClient.instance().get_concept(**locals())


@assert_auth
@hashable_lru(maxsize=3)
def get_all_securities(types=[], date=None):
    """
    获取平台支持的所有股票、基金、指数、期货信息

    :param types list: 用来过滤securities的类型, list元素可选: ‘stock’, ‘fund’, ‘index’, ‘futures’, ‘etf’, ‘lof’, ‘fja’, ‘fjb’. types为空时返回所有股票, 不包括基金,指数和期货
    :param date 日期, 一个字符串或者 datetime.datetime/datetime.date 对象, 用于获取某日期还在上市的股票信息. 默认值为 None, 表示获取所有日期的股票信息
    :return pandas.DataFrame
    """
    date = to_date_str(date)
    return JQDataClient.instance().get_all_securities(**locals())


@assert_auth
@hashable_lru(maxsize=3)
def get_security_info(code, date=None):
    """
    获取股票/基金/指数的信息

    :param code 证券代码
    :param date 查询日期: 日期字符串/date对象/datetime对象, 注意传入datetime对象时忽略日内时间
    :return Security
    """
    assert code, "code is required"
    date = to_date_str(date)
    result = JQDataClient.instance().get_security_info(**locals())
    if result:
        return Security(**result)


@assert_auth
@hashable_lru(maxsize=3)
def get_all_trade_days():
    """
    获取所有交易日

    :return 包含所有交易日的 numpy.ndarray, 每个元素为一个 datetime.date 类型.
    """
    data = JQDataClient.instance().get_all_trade_days()
    if str(data.dtype) != "object":
        data = data.astype(datetime.datetime)
    return data


@assert_auth
@hashable_lru(maxsize=3)
def get_trade_days(start_date=None, end_date=None, count=None):
    """
    获取指定日期范围内的所有交易日

    :return numpy.ndarray, 包含指定的 start_date 和 end_date, 默认返回至 datatime.date.today() 的所有交易日
    """
    start_date = to_date_str(start_date)
    end_date = to_date_str(end_date)
    data = JQDataClient.instance().get_trade_days(**locals())
    if str(data.dtype) != "object":
        data = data.astype(datetime.datetime)
    return data


@assert_auth
@hashable_lru(maxsize=3)
def get_money_flow(security_list, start_date=None, end_date=None, fields=None, count=None):
    """
    获取一只或者多只股票在一个时间段内的资金流向数据

    :param security_list 一只股票代码或者一个股票代码的 list
    :param start_date 开始日期, 与 count 二选一, 不可同时使用, 一个字符串或者 datetime.datetime/datetime.date 对象, 默认为平台提供的数据的最早日期
    :param end_date 结束日期, 一个字符串或者 datetime.date/datetime.datetime 对象, 默认为 datetime.date.today()
    :param count 数量, 与 start_date 二选一，不可同时使用, 必须大于 0. 表示返回 end_date 之前 count 个交易日的数据, 包含 end_date
    :param fields 字段名或者 list, 可选
    :return pandas.DataFrame
    """
    assert security_list, "security_list is required"
    security_list = convert_security(security_list)
    start_date = to_date_str(start_date)
    end_date = to_date_str(end_date)
    return JQDataClient.instance().get_money_flow(**locals())


@assert_auth
@hashable_lru(maxsize=3)
def get_mtss(security_list, start_date=None, end_date=None, fields=None, count=None):
    """
    获取一只或者多只股票在一个时间段内的融资融券信息

    :param security_list 一只股票代码或者一个股票代码的 list
    :param start_date 开始日期, 与 count 二选一, 不可同时使用.
    :param end_date 结束日期, 一个字符串或者 datetime.date/datetime.datetime 对象, 默认为 datetime.date.today()
    :param count 数量, 与 start_date 二选一，不可同时使用, 必须大于 0. 表示返回 end_date 之前 count 个交易日的数据, 包含 end_date
    :param fields 字段名或者 list, 可选. 默认为 None, 表示取全部字段
    :return pandas.DataFrame
    """
    assert (not start_date) ^ (not count), "(start_date, count) only one param is required"
    start_date = to_date_str(start_date)
    end_date = to_date_str(end_date)
    security_list = convert_security(security_list)
    return JQDataClient.instance().get_mtss(**locals())


@assert_auth
@hashable_lru(maxsize=3)
def get_margincash_stocks(date=None):
    """
    返回上交所、深交所最近一次披露的的可融资标的列表的list
    :return: list
    """
    date = to_date_str(date)
    return JQDataClient.instance().get_margincash_stocks(**locals())


@assert_auth
@hashable_lru(maxsize=3)
def get_marginsec_stocks(date=None):
    """
    返回上交所、深交所最近一次披露的的可融券标的列表的list
    :return:list
    """
    date = to_date_str(date)
    return JQDataClient.instance().get_marginsec_stocks(**locals())


@assert_auth
def get_future_contracts(underlying_symbol, date=None):
    """
    获取某期货品种在策略当前日期的可交易合约标的列表

    :param security 期货合约品种，如 ‘AG’(白银)
    :return 某期货品种在策略当前日期的可交易合约标的列表
    """
    assert underlying_symbol, "underlying_symbol is required"
    dt = to_date_str(date)
    return JQDataClient.instance().get_future_contracts(underlying_symbol=underlying_symbol, dt=dt)


@assert_auth
def get_dominant_future(underlying_symbol, date=None):
    """
    获取主力合约对应的标的

    :param underlying_symbol 期货合约品种，如 ‘AG’(白银)
    :return 主力合约对应的期货合约
    """
    dt = to_date_str(date)
    return JQDataClient.instance().get_dominant_future(underlying_symbol=underlying_symbol, dt=dt)


@assert_auth
def get_ticks(security, start_dt=None, end_dt=None, count=None, fields=None, skip=True, df=True):
    """
    获取tick数据
    :param security: 股票or期货标的代码,仅限单只
    :param start_dt: 开始日期
    :param end_dt: 截止日期
    :param count: 统计个数
    :param fields: 期货：[time current high low volume money position a1_v a1_p b1_v b1_p]
                    股票：[time current high low volume money a1_v-a5_v a1_p-a5_p b1_v-b5_v b1_p-b5_p]
                    为None时，默认返回对应类型的所有字段
    :param skip: 是否过滤掉无成交的tick
    :param df: 默认为True，传入单个标的返回的是一个dataframe, 当df=False的时候，当单个标的的时候，返回一个np.ndarray
    :return:
    """
    start_dt = to_date_str(start_dt)
    end_dt = to_date_str(end_dt)
    return JQDataClient.instance().get_ticks(**locals())


@assert_auth
def get_ticks_engine(security, start_dt=None, end_dt=None, count=None, fields=None):
    start_dt = to_date_str(start_dt)
    end_dt = to_date_str(end_dt)
    return JQDataClient.instance().get_ticks_engine(**locals())


@assert_auth
def get_baidu_factor(category=None, day=None, stock=None, province=None):
    """
    获取百度因子搜索量数据
    :param category:数据类别，中证800的数据类别为"csi800"
    :param stock: 一只股票或一个股票list。如果为空，则包含中证800所有的成分股。
    :param day:日期，date、datetime或字符串类型。如果day为空，则返回最新的数据。
    :param province:省份名称或省份代码，如北京或110000。如果为空，则返回PC端和手机端的数据汇总。
    如果不为空，则返回指定省份的数据。
    :return:
    """

    day = to_date_str(day)
    stock = normal_security_code(stock)
    return JQDataClient.instance().get_baidu_factor(**locals())


@assert_auth
def normalize_code(code):
    """
    归一化证券代码

    :param code 如000001
    :return 证券代码的全称 如000001.XSHE
    """
    return JQDataClient.instance().normalize_code(**locals())


@assert_auth
@hashable_lru(maxsize=3)
def get_factor_values(securities, factors, start_date=None, end_date=None, count=None):
    """
    获取因子数据

    返回字典类型
    :return:
    """

    securities = convert_security(securities)
    start_date = to_date_str(start_date)
    end_date = to_date_str(end_date)
    if (not count) and (not start_date):
        start_date = "2015-01-01"
    if count and start_date:
        raise ParamsError("(start_date, count) only one param is required")
    return JQDataClient.instance().get_factor_values(**locals())


@assert_auth
@hashable_lru(maxsize=3)
def get_index_weights(index_id, date=None):
    """
    获取指数成分股权重

    :param index_id 必选参数，代表指数的标准形式代码
    :param date 可选参数， 查询权重信息的日期
    :return
    """
    assert index_id, "index_id is required"
    date = to_date_str(date)
    return JQDataClient.instance().get_index_weights(**locals())


@assert_auth
@hashable_lru(maxsize=3)
def get_industry(security, date=None):
    """
    查询股票所属行业

    :param security 标的代码
    :date 查询的日期，默认为空
    :return 字典格式
    """
    assert security, "security is required"
    security = convert_security(security)
    date = to_date_str(date)
    return JQDataClient.instance().get_industry(**locals())


@assert_auth
def get_bars(security, count, unit="1d", fields=("date", "open", "high", "low", "close"), include_now=False, end_dt=None,
             fq_ref_date=None, df=True):
    """
    获取历史数据(包含快照数据), 可查询单个标的多个数据字段

    :param security 股票代码
    :param count 大于0的整数，表示获取bar的个数。如果行情数据的bar不足count个，返回的长度则小于count个数。
    :param unit bar的时间单位, 支持如下周期：'1m', '5m', '15m', '30m', '60m', '120m', '1d', '1w', '1M'。'1w' 表示一周，‘1M' 表示一月。
    :param fields 获取数据的字段， 支持如下值：'date', 'open', 'close', 'high', 'low', 'volume', 'money'
    :param include_now 取值True 或者False。 表示是否包含当前bar, 比如策略时间是9:33，unit参数为5m， 如果 include_now=True,则返回9:30-9:33这个分钟 bar。
    :param end_dt: 查询的截止时间
    :param fq_ref_date: 复权基准日期，为None时为不复权数据
    :param df: 默认为True，传入单个标的返回的是一个dataframe，传入多个标的返回的是一个multi-index dataframe
            当df=False的时候，当单个标的的时候，返回一个np.ndarray，多个标的返回一个字典，key是code，value是np.array；
    :return numpy.ndarray格式
    """
    assert security, "security is required"
    security = convert_security(security)
    end_dt = to_date_str(end_dt)
    fq_ref_date = to_date_str(fq_ref_date)
    return JQDataClient.instance().get_bars(**locals())


@assert_auth
def get_bars_engine(security, count, unit="1d", fields=("open", "high", "low", "close"), include_now=False, end_dt=None,
                    fq_ref_date=None):
    security = convert_security(security)
    fq_ref_date = to_date_str(fq_ref_date)
    if not (isinstance(security, six.string_types) or isinstance(security, (tuple, list))):
        raise Exception('security 必须是字符串 或者 字符串数组')
    end_dt = to_date_str(end_dt)
    return JQDataClient.instance().get_bars_engine(**locals())


# @assert_auth
# def get_current_tick2(security):
#     """
#     获取最新的 tick 数据
#     :param security 标的代码
#     :return:
#     """
#     assert security, "security is required"
#     security = convert_security(security)
#     return JQDataClient.instance().get_current_tick(**locals())

def get_current_tick(security):
    """
    获取最新的 tick 数据

    :param security 标的代码
    :return:
    """
    if not JQDataClient.instance() or JQDataClient.instance().get_http_token() == "":
        print("run jqdatasdk.auth first")
        return

    if isinstance(security, six.string_types):
        security = [security]
    elif isinstance(security, Security):
        security = [str(security)]
    res = request_data(security)
    if not res or res.text == "":
        return None
    content = res.text
    if content[:5] == 'error':
        if content in ["error: token无效，请重新获取","error: token过期，请重新获取"]:
            JQDataClient.instance().set_http_token()
            res = request_data(security)  # 重试一次
            if not res or res.text == "":
                return None
            content = res.text
            if content[:5] == 'error':
                raise Exception(content)
        else:
            raise Exception(content)

    stock_tick_fields = ['datetime', 'current', 'high', 'low', 'volume', 'money',
                         'a1_p', 'a1_v', 'a2_p', 'a2_v', 'a3_p', 'a3_v', 'a4_p', 'a4_v', 'a5_p', 'a5_v',
                         'b1_p', 'b1_v', 'b2_p', 'b2_v', 'b3_p', 'b3_v', 'b4_p', 'b4_v', 'b5_p', 'b5_v']
    option_tick_fields = ['datetime', 'current', 'high', 'low', 'volume', 'money', 'position',
                         'a1_v', 'a2_v', 'a3_v', 'a4_v', 'a5_v', 'a1_p', 'a2_p', 'a3_p', 'a4_p', 'a5_p',
                         'b1_v', 'b2_v', 'b3_v', 'b4_v', 'b5_v', 'b1_p', 'b2_p', 'b3_p', 'b4_p', 'b5_p']
    future_tick_fields = ['datetime', 'current', 'high', 'low', 'volume', 'money', 'position', 'a1_p', 'a1_v', 'b1_p', 'b1_v']
    index_tick_fields = ['datetime', 'current', 'high', 'low', 'volume', 'money']

    tick_fields_list = [stock_tick_fields,future_tick_fields,stock_tick_fields,index_tick_fields,option_tick_fields]
    # content[0]返回数据第一个字符为标的类型  "0":"stock","1":"future","2":"fund","3":"index","4":"option"
    tick_fields = tick_fields_list[int(content[0])]

    str2time = lambda x: datetime.datetime.strptime(x, '%Y%m%d%H%M%S.%f') if x else pd.NaT
    data = StringIO(content)
    data.seek(1)  # 跳过第一个字符，从第二个开始取数据
    df = pd.read_csv(data, index_col=0, converters={"datetime": str2time})
    df = df[tick_fields]
    if len(security) <= 1:
        df.index = [0]
    return df

def request_data(security):
    http_token = JQDataClient.instance().get_http_token()
    codes = ",".join(security)
    headers = {
        'Accept-Encoding': 'gzip, deflate',
        'Content-Type': 'application/json',
        'Connection': 'keep-alive'
    }
    body = {
        "method": "get_current_ticks2",
        "token": http_token,
        "code": codes
    }
    data_api_url = JQDataClient.instance().get_data_api_url()
    res = requests.post(data_api_url, data = json.dumps(body), headers = headers)
    return res

@assert_auth
def get_current_tick_engine(security):
    assert security, "security is required"
    if not (isinstance(security, six.string_types) or isinstance(security, (tuple, list))):
        raise Exception('security 必须是字符串 或者 字符串数组')
    security = convert_security(security)
    return JQDataClient.instance().get_current_tick_engine(**locals())


@assert_auth
@hashable_lru(maxsize=3)
def get_fund_info(security, date=None):
    """
    基金基础信息数据接口

    :param security 基金代码
    :param date 查询日期
    :return 字典格式
    """
    assert security, "security is required"
    security = convert_security(security)
    date = to_date_str(date)
    return JQDataClient.instance().get_fund_info(**locals())


@assert_auth
def get_query_count(field=None):
    """
    查询当日可请求条数/剩余请求条数

    :param field
        默认为None，返回当日可请求条数/剩余请求条数，字典格式
        "total", 返回当日可请求条数, int格式
        "spare", 返回当日剩余请求条数, int格式
    """
    assert field in ["total", "spare", None], "field参数必须为total,spare,None中的一个"
    return JQDataClient.instance().get_query_count(**locals())


@assert_auth
def history_engine(end_dt, count, unit='1d', field='avg', security_list=None,
                   df=True, skip_paused=False, fq='pre', pre_factor_ref_date=None):
    security_list = convert_security(security_list)
    end_dt = to_date_str(end_dt)
    pre_factor_ref_date = to_date_str(pre_factor_ref_date)
    return JQDataClient.instance().history_engine(**locals())


@assert_auth
def attribute_history_engine(end_dt, security, count, unit='1d',
                             fields=('open', 'close', 'high', 'low', 'volume', 'money'),
                             skip_paused=True,
                             df=True,
                             fq='pre',
                             pre_factor_ref_date=None):
    security = convert_security(security)
    end_dt = to_date_str(end_dt)
    pre_factor_ref_date = to_date_str(pre_factor_ref_date)
    return JQDataClient.instance().attribute_history_engine(**locals())


@assert_auth
def get_daily_info_engine(security, date=None):
    """
    查询复权因子

    :param security 代码，string or list
    :param date 日期，默认空，取当日数据
    :return 一个股票代码返回str，多个股票代码返回dict，
            str: 日期，状态，复权因子
    """
    security = convert_security(security)
    date = to_date_str(date)
    return JQDataClient.instance().get_daily_info_engine(**locals())


@assert_auth
def get_last_price(codes):
    """获取标的的最新价格
    :param codes 一支证券代码或者一个证券代码的list
    :return 返回dict对象，如果传入的证券没有数据，不包含返回的dict
    """
    codes = convert_security(codes)
    return JQDataClient.instance().get_last_price(**locals())


@assert_auth
@hashable_lru(maxsize=3)
def get_factor_effect(security, start_date, end_date, period, factor, group_num=5):
    """获取因子分层回测效果
    以因子值升序分组股票, 以period为周期，获取每组股票收益情况
    :param security 一支指数代码
    :param start_date 开始日期
    :param end_date 结束日期
    :param period 周期，xD, xW, xM > 几日, 几周, 几月
    :param factor 因子名称
    :param group_num 分组数；default 5
    :return dataframe
    """
    security = convert_security(security)
    start_date = to_date_str(start_date)
    end_date = to_date_str(end_date)
    assert group_num > 0, "group_num must be a positive numbe"
    assert isinstance(security, six.string_types), "security must be a inde code" 
    assert period[-1] in ["D", "W", "M"], "period must be end with one of (\"D\", \"W\", \"M\")"
    return JQDataClient.instance().get_factor_effect(**locals())

@assert_auth
def get_data(api_name, **kwargs):
    """通用接口
    获取非run_query, alpha因子, technical_analysis查询的数据
    :param apis_name 接口名称
    :param kwargs 传入参数，且必须是关键字参数
    """
    assert api_name
    return JQDataClient.instance().get_data(api_name=api_name, args=kwargs)


@assert_auth
@hashable_lru(maxsize=3)
def get_all_factors():
    return JQDataClient.instance().get_all_factors(**locals())

@assert_auth
@hashable_lru(maxsize=3)
def get_call_auction(security, start_date=None, end_date=None, fields=None):
    """ 获取指定时间区间内集合竞价时的tick数据
    Args:
        security: 标的代码或者标的代码组成的列表
        start_date: 开始日期，一个时间字符串, 比如"2019-01-01"
        end_date: 结束日期，一个时间字符, 比如"2019-02-01"
        fields:选择要获取的行情字段(类似tick数据的每一个字段)，字段介绍如下:
            字段名         说明              字段类型
            time	      时间	            datetime
            current	      当前价	            float
            volume	      累计成交量（股）  	float
            money	      累计成交额	        float
            b1_v~b5_v	  五档买量          	float
            b1_p~b5_v	  五档买价	        float
            a1_v~a5_v	  五档卖量	        float
            a1_p~a5_p	  五档卖价	        float

        比如:
            get_call_auction(['000001.XSHE', '000002.XSHE'], satrt_date='2019-01-01', end_date='2019-10-10',
                              fields=['time', 'current', 'a1_v', 'b1_v'])

    Return:
        返回一个dataframe，索引为pandas默认的整数索引

    Notice:
        1. field为None的时候，默认获取全部字段
            get_call_auction('000001.XSHE','2019-08-10','2019-08-12')
        2. start_date和end_date不能为None，否则将抛出异常
    """
    return JQDataClient.instance().get_call_auction(**locals())

def read_file(path):
    """
    读取文件
    """
    with open(path, 'rb') as f:
        return f.read()


def write_file(path, content, append=False):
    """
    写入文件
    """
    if isinstance(content, six.text_type):
        content = content.encode('utf-8')
    with open(path, 'ab' if append else 'wb') as f:
        return f.write(content)


__all__ = ["normalize_code", "attribute_history_engine", "history_engine",]

def _collect_func():
    funcs = []
    for func in globals().keys():
        if func.startswith("get"):
            funcs.append(func)
    return funcs

__all__.extend(_collect_func())

del _collect_func
